# encoding: utf-8

# author: BrikerMan
# contact: eliyar917@gmail.com
# blog: https://eliyar.biz

# file: __init__.py
# time: 2019-05-20 11:34

from kashgari.tasks import labeling
from kashgari.tasks import classification

if __name__ == "__main__":
    print("Hello world")
