# encoding: utf-8

# author: BrikerMan
# contact: eliyar917@gmail.com
# blog: https://eliyar.biz

# file: __init__.py.py
# time: 2019-05-31 19:29


if __name__ == "__main__":
    print("Hello world")
